//
//  QHVCRTCClient.h
//  QHVCRTCKit
//
//  Created by niezhiqiang on 2019/11/20.
//  Copyright © 2019 qihoo360. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

typedef enum
{
    QHVCFrameType_AAC = 3,
    QHVCFrameType_OPUS,
    QHVCFrameType_PCM_S16LE,
    QHVCFrameType_PCM_ULAW,
    QHVCFrameType_PCM_ALAW
}QHVCRTCFrameType;

@interface QHVCRTCAudioParams : NSObject

@property (nonatomic, assign) QHVCRTCFrameType frameType;
@property (nonatomic, assign) int sampleRate;
@property (nonatomic, assign) int channel;
@property (nonatomic, assign) int sampleDepth;

@end

@protocol QHVCRTCDelegate <NSObject>
@optional

/// 本地音频数据回调
/// @param sessionId 会话ID
/// @param data 音频帧数据
/// @param frameSize 帧大小
/// @param timestamp 时间戳
- (void)onRTCAudioCallback:(NSString *)sessionId
                      data:(const uint8_t* _Nullable)data
                 frameSize:(int)frameSize
                 timestamp:(uint64_t)timestamp;

/// 音量回调（0-255），默认200ms回调一次，可通过 setAudioVolumeInterval 接口自定义设置
/// @param localVolume 本地音量
/// @param remoteVolume 远端音量
- (void)onVolumeCallback:(int)localVolume remote:(int)remoteVolume;

@end

@interface QHVCRTCClient : NSObject

@property (nonatomic, weak) id<QHVCRTCDelegate> delegate;
@property (nonatomic, readonly) BOOL isMutedLocalAudio;
@property (nonatomic, readonly) BOOL isMutedRemoteAudio;
@property (nonatomic, readonly) QHVCRTCAudioParams* audioParamsIn;
@property (nonatomic, readonly) QHVCRTCAudioParams* audioParamsOut;
@property (nonatomic, readonly) NSString* sessionId;

+ (instancetype)sharedInstance;

/// 启动
/// @param sessionId 会话ID，需保持每次会话ID唯一且不重复
/// @param audioParamsIn 音频输入参数
/// @param audioParamsOut 音频输出参数
/// @param options 扩展配置，例如：
/// @{
///     @"preferAudioPacketDuration": @(20),  //音频包时长，默认20ms
///  }
- (void)startWithSessionId:(NSString *)sessionId
             audioParamsIn:(QHVCRTCAudioParams *)audioParamsIn
            audioParamsOut:(QHVCRTCAudioParams *)audioParamsOut
                   options:(NSDictionary *)options;

/// 停止
- (void)stop;

/// 本地音频流静音
/// @param muted 是否静音
- (void)muteLocalAudioStream:(BOOL)muted;

/// 远端音频流静音
/// @param muted 是否静音
- (void)muteRemoteAudioStream:(BOOL)muted;

/// 设置本地音频流音量放大倍数
/// @param multiples 放大倍数，默认1.0
- (void)setLocalAudioVolumeMultiples:(float)multiples;

/// 设置本地音频流音调
/// @param pitch 音调，默认原声1.0，例如女声1.6，男声0.8
- (void)setLocalAudioPitch:(float)pitch;

/// 设置音量回调间隔，默认300ms，正常取值范围[200,3000]
/// @param interval 回调间隔
- (void)setAudioVolumeInterval:(int)interval;

/// 发送远端音频数据
/// @param frameData 音频数据
/// @param frameSize 数据大小
/// @param timestamp 时间戳
- (void)sendAudioFrame:(const uint8_t *)frameData
             frameSize:(int)frameSize
             timestamp:(uint64_t)timestamp;


@end

NS_ASSUME_NONNULL_END
